"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.default = void 0;

var _lodash = require("lodash");

var _flow = _interopRequireDefault(require("./api/flow"));

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

function asyncGeneratorStep(gen, resolve, reject, _next, _throw, key, arg) { try { var info = gen[key](arg); var value = info.value; } catch (error) { reject(error); return; } if (info.done) { resolve(value); } else { Promise.resolve(value).then(_next, _throw); } }

function _asyncToGenerator(fn) { return function () { var self = this, args = arguments; return new Promise(function (resolve, reject) { var gen = fn.apply(self, args); function _next(value) { asyncGeneratorStep(gen, resolve, reject, _next, _throw, "next", value); } function _throw(err) { asyncGeneratorStep(gen, resolve, reject, _next, _throw, "throw", err); } _next(undefined); }); }; }

function _classCallCheck(instance, Constructor) { if (!(instance instanceof Constructor)) { throw new TypeError("Cannot call a class as a function"); } }

function _defineProperties(target, props) { for (var i = 0; i < props.length; i++) { var descriptor = props[i]; descriptor.enumerable = descriptor.enumerable || false; descriptor.configurable = true; if ("value" in descriptor) descriptor.writable = true; Object.defineProperty(target, descriptor.key, descriptor); } }

function _createClass(Constructor, protoProps, staticProps) { if (protoProps) _defineProperties(Constructor.prototype, protoProps); if (staticProps) _defineProperties(Constructor, staticProps); return Constructor; }

var Ancestry =
/*#__PURE__*/
function () {
  _createClass(Ancestry, null, [{
    key: "inflate",
    value: function () {
      var _inflate = _asyncToGenerator(
      /*#__PURE__*/
      regeneratorRuntime.mark(function _callee(deflated, axios) {
        var flowAPI, flows, flowMap, _anc, _loop, d;

        return regeneratorRuntime.wrap(function _callee$(_context) {
          while (1) {
            switch (_context.prev = _context.next) {
              case 0:
                if ((0, _lodash.isArray)(deflated)) {
                  _context.next = 2;
                  break;
                }

                return _context.abrupt("return", deflated);

              case 2:
                flowAPI = new _flow.default(axios);
                _context.next = 5;
                return flowAPI.loadMany(deflated.map(function (level) {
                  return level.flowId;
                }));

              case 5:
                flows = _context.sent;
                flowMap = (0, _lodash.keyBy)(flows, '_id');
                _anc = [];

                _loop = function _loop(d) {
                  var _deflated$d = deflated[d],
                      flowId = _deflated$d.flowId,
                      conditionalId = _deflated$d.conditionalId;
                  var flow = flowMap[flowId];

                  _anc.push(flow);

                  if (conditionalId) {
                    _anc.push(flow.steps.find(function (step) {
                      return step._id === conditionalId;
                    }));
                  }
                };

                for (d = deflated.length - 1; d >= 0; d--) {
                  _loop(d);
                }

                return _context.abrupt("return", new Ancestry(_anc));

              case 11:
              case "end":
                return _context.stop();
            }
          }
        }, _callee, this);
      }));

      return function inflate(_x, _x2) {
        return _inflate.apply(this, arguments);
      };
    }()
  }]);

  function Ancestry(ancestry) {
    _classCallCheck(this, Ancestry);

    this.ancestry = ancestry;
    this.level = [];
    this.walk(ancestry);
  }

  _createClass(Ancestry, [{
    key: "walk",
    value: function walk(ancestry) {
      for (var i = ancestry.length - 1; i >= 0; i--) {
        var item = ancestry[i];
        var type = this.getType(item);

        switch (type) {
          case 'conditional':
            var flowContainingConditional = ancestry[i - 1];

            if (this.getType(flowContainingConditional) !== 'flow') {
              throw new Error('Expected flow in array before conditional');
            }

            this.level.push(this.createSummary(flowContainingConditional, item));
            i--;
            break;

          case 'flow':
            this.level.push(this.createSummary(item));
            break;

          default:
            throw new Error('Unknown object type in ancestry');
        }
      }
    }
  }, {
    key: "getType",
    value: function getType(item) {
      return item.type || 'flow';
    }
  }, {
    key: "createSummary",
    value: function createSummary(flow, conditional) {
      var summary = {
        flow: flow
      };

      if (conditional) {
        summary.conditional = conditional;
        flow.steps.forEach(function (step, index) {
          if (step._id === conditional._id) {
            summary.step = index;
          }
        });
      }

      return summary;
    }
  }, {
    key: "refresh",
    value: function refresh(flowMap, flow, clone) {
      if (flow) {
        flowMap[flow._id] = flow;
      }

      this.level.forEach(function (summary) {
        summary.flow = flowMap[summary.flow._id];

        if (summary.conditional) {
          summary.flow.steps.forEach(function (step, index) {
            if (step._id === summary.conditional._id) {
              summary.conditional = step;
              summary.step = index;
            }
          });
        }
      });

      if (clone) {
        this.level = (0, _lodash.cloneDeep)(this.level);
      }
    }
  }, {
    key: "getCurrentSummary",
    value: function getCurrentSummary() {
      return this.level[0];
    }
  }, {
    key: "getCurrentFlow",
    value: function getCurrentFlow() {
      return this.level[0].flow;
    }
  }, {
    key: "getRootSummary",
    value: function getRootSummary() {
      return this.level[this.level.length - 1];
    }
  }, {
    key: "getRootFlow",
    value: function getRootFlow() {
      return this.level[this.level.length - 1].flow;
    }
  }, {
    key: "forEachStep",
    value: function forEachStep(x, y) {
      var position;
      var fn;

      if ((0, _lodash.isFunction)(x)) {
        fn = x;
        position = this.level[0].flow.steps.length;
      } else {
        fn = y;
        position = x === null || x === undefined ? this.level[0].flow.steps.length - 1 : x;
      }

      for (var i = 0; i < this.level.length; i++) {
        var _this$level$i = this.level[i],
            flow = _this$level$i.flow,
            step = _this$level$i.step;
        var start = step || Math.min(position - 1, flow.steps.length);

        for (; start >= 0; start--) {
          fn({
            ancestry: this,
            level: i,
            flow: flow,
            position: start,
            step: flow.steps[start]
          });
        }
      }
    }
  }, {
    key: "deflate",
    value: function deflate() {
      return this.level.map(function (level) {
        var deflated = {
          flowId: level.flow._id
        };
        if (level.conditional) deflated.conditionalId = level.conditional._id;
        return deflated;
      });
    }
  }]);

  return Ancestry;
}();

exports.default = Ancestry;
//# sourceMappingURL=ancestry-util.js.map
