"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.getWorkflowContextUtil = getWorkflowContextUtil;
exports.getWorkflowRelevantContexts = getWorkflowRelevantContexts;
exports.matchesReturnType = matchesReturnType;

var _lodash = require("lodash");

var _ancestryUtil = _interopRequireDefault(require("../ancestry-util"));

var _contextUtils = _interopRequireWildcard(require("../data-dictionary/context-utils"));

var _returnTypes = require("../data-dictionary/return-types");

var _workflowContext = _interopRequireDefault(require("./workflow-context"));

var _workflowContexts = _interopRequireDefault(require("./workflow-contexts"));

function _interopRequireWildcard(obj) { if (obj && obj.__esModule) { return obj; } else { var newObj = {}; if (obj != null) { for (var key in obj) { if (Object.prototype.hasOwnProperty.call(obj, key)) { var desc = Object.defineProperty && Object.getOwnPropertyDescriptor ? Object.getOwnPropertyDescriptor(obj, key) : {}; if (desc.get || desc.set) { Object.defineProperty(newObj, key, desc); } else { newObj[key] = obj[key]; } } } } newObj.default = obj; return newObj; } }

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

function _defineProperty(obj, key, value) { if (key in obj) { Object.defineProperty(obj, key, { value: value, enumerable: true, configurable: true, writable: true }); } else { obj[key] = value; } return obj; }

// TODO: we need to connect to the real step definition map
var stepDefinitionMap = {
  conditional: {},
  formfill: {
    contexts: _defineProperty({}, _returnTypes.FORM, 'meta.form')
  },
  'form-approval': {},
  'form-acknowledge': {},
  integration: {},
  notification: {}
  /**
   *
   * @param {*} _ancestry
   * @param {*} position
   * @param {*} returnTypes
   */

};

function getWorkflowContextUtil(ancestry, position, returnTypes, axios) {
  var scope = {
    ancestry: ancestry,
    position: position
  };
  if (returnTypes) scope.returnTypes = returnTypes;
  if (axios) scope.axios = axios;
  var ctx = new _contextUtils.ContextUtil(scope);
  ctx.addContext(_workflowContext.default);
  ctx.addContext(_workflowContexts.default);
  var root = ctx.getRoot();
  root.addContext(_workflowContexts.default, {
    ancestry: ancestry,
    position: position
  });
  return ctx;
}
/**
 * Get the contexts relevant to previous steps in the passed workflow
 * @param {Array} _ancestry The raw array of steps and conditionals
 * @param {Number} position The position in the current flow from which to start
 * @param {Array} returnTypes Types of values that the contexts should return
 * @returns {Array} deflated contexts
 */


function getWorkflowRelevantContexts(_ancestry, position, returnTypes) {
  var ancestry = new _ancestryUtil.default(_ancestry);
  var contextMap = _contextUtils.default.contextMap,
      getDeflatedContextType = _contextUtils.default.getDeflatedContextType;
  var relevant = [];
  ancestry.forEachStep(position, function (stepLocation) {
    var stepType = (0, _lodash.get)(stepLocation, 'step.type');
    if (!stepType) return;
    var stepContextMap = stepDefinitionMap[stepType].contexts;
    if (!stepContextMap) return;
    (0, _lodash.forEach)(stepContextMap, function (path, type) {
      var deflated = (0, _lodash.get)(stepLocation.step, path);
      var Context = contextMap[getDeflatedContextType(deflated)];

      if (matchesReturnType(Context, returnTypes)) {
        relevant.push({
          context: deflated,
          step: stepLocation
        });
      }
    });
  });
  return relevant;
}
/**
 * Returns true if the Context class can return one of the types in the
 * returnType array
 * @param {Object} Context The context class
 * @param {Array} returnTypes An array of return types to match or '*' for all
 */


function matchesReturnType(Context, returnTypes) {
  return Context.returnTypes === '*' || Context.returnTypes.some(function (type) {
    return returnTypes.includes(type);
  });
}
//# sourceMappingURL=utils.js.map
