"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.coerce = coerce;
exports.coercionNotSupported = exports.COERCIONS = void 0;

var _lodash = _interopRequireDefault(require("lodash"));

var _coercionNotSupportedError = _interopRequireDefault(require("./coercion-not-supported-error"));

var booleanDataType = _interopRequireWildcard(require("./data-types/boolean"));

var groupDataType = _interopRequireWildcard(require("./data-types/group"));

var numberDataType = _interopRequireWildcard(require("./data-types/number"));

var userDataType = _interopRequireWildcard(require("./data-types/user"));

var textDataType = _interopRequireWildcard(require("./data-types/text"));

function _interopRequireWildcard(obj) { if (obj && obj.__esModule) { return obj; } else { var newObj = {}; if (obj != null) { for (var key in obj) { if (Object.prototype.hasOwnProperty.call(obj, key)) { var desc = Object.defineProperty && Object.getOwnPropertyDescriptor ? Object.getOwnPropertyDescriptor(obj, key) : {}; if (desc.get || desc.set) { Object.defineProperty(newObj, key, desc); } else { newObj[key] = obj[key]; } } } } newObj.default = obj; return newObj; } }

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

var DATA_TYPES = [booleanDataType, groupDataType, numberDataType, userDataType, textDataType];

var COERCIONS = _lodash.default.reduce(DATA_TYPES, function (accumulator, dataType) {
  return _lodash.default.set(accumulator, dataType.TYPE, dataType.COERCIONS);
}, {});

exports.COERCIONS = COERCIONS;

function coerce(baseType, requestedCoerceType, value) {
  var coercionFn = _lodash.default.get(COERCIONS, [baseType, requestedCoerceType], coercionNotSupported(baseType, requestedCoerceType));

  return coercionFn(value);
}

var coercionNotSupported = _lodash.default.curry(function throwCoercionNotSupportedError(baseType, requestedCoerceType, value) {
  throw new _coercionNotSupportedError.default(baseType, requestedCoerceType, value);
});

exports.coercionNotSupported = coercionNotSupported;
//# sourceMappingURL=coerce.js.map
