"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.getOperator = exports.names = exports.IS_TRUE = exports.IS_ONE_OF = exports.IS_NOT_EMPTY = exports.IS_NOT = exports.IS_LESS_THAN = exports.IS_GREATER_THAN = exports.IS_FALSE = exports.IS_EMPTY = exports.IS = exports.HAS_SELECTED = exports.HAS_NOT_SELECTED = exports.ENDS_WITH = exports.DOES_NOT_END_WITH = exports.DOES_NOT_CONTAIN = exports.DOES_NOT_BEGIN_WITH = exports.CONTAINS = exports.BEGINS_WITH = exports.isUnary = exports.isBinary = exports.OPERATOR_TYPES = void 0;

var _lodash = _interopRequireDefault(require("lodash"));

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

/* Copyright © 2016 Kuali, Inc. - All Rights Reserved
 * You may use and modify this code under the terms of the Kuali, Inc.
 * Pre-Release License Agreement. You may not distribute it.
 *
 * You should have received a copy of the Kuali, Inc. Pre-Release License
 * Agreement with this file. If not, please write to license@kuali.co.
 */
var OPERATOR_TYPES = {
  BINARY: 'binary',
  UNARY: 'unary'
};
exports.OPERATOR_TYPES = OPERATOR_TYPES;

var makeOperator = _lodash.default.curry(function (type, operator) {
  return {
    operator: operator,
    type: type,
    toString: _lodash.default.constant(operator)
  };
});

var binary = makeOperator(OPERATOR_TYPES.BINARY);
var unary = makeOperator(OPERATOR_TYPES.UNARY);

var isBinary = function isBinary(operator) {
  return operator.type === OPERATOR_TYPES.BINARY;
};

exports.isBinary = isBinary;

var isUnary = function isUnary(operator) {
  return operator.type === OPERATOR_TYPES.UNARY;
};

exports.isUnary = isUnary;
var BEGINS_WITH = binary('begins with');
exports.BEGINS_WITH = BEGINS_WITH;
var CONTAINS = binary('contains');
exports.CONTAINS = CONTAINS;
var DOES_NOT_BEGIN_WITH = binary('does not begin with');
exports.DOES_NOT_BEGIN_WITH = DOES_NOT_BEGIN_WITH;
var DOES_NOT_CONTAIN = binary('does not contain');
exports.DOES_NOT_CONTAIN = DOES_NOT_CONTAIN;
var DOES_NOT_END_WITH = binary('does not end with');
exports.DOES_NOT_END_WITH = DOES_NOT_END_WITH;
var ENDS_WITH = binary('ends with');
exports.ENDS_WITH = ENDS_WITH;
var HAS_NOT_SELECTED = binary('has not selected');
exports.HAS_NOT_SELECTED = HAS_NOT_SELECTED;
var HAS_SELECTED = binary('has selected');
exports.HAS_SELECTED = HAS_SELECTED;
var IS = binary('is');
exports.IS = IS;
var IS_EMPTY = unary('is empty');
exports.IS_EMPTY = IS_EMPTY;
var IS_FALSE = unary('is false');
exports.IS_FALSE = IS_FALSE;
var IS_GREATER_THAN = binary('is greater than');
exports.IS_GREATER_THAN = IS_GREATER_THAN;
var IS_LESS_THAN = binary('is less than');
exports.IS_LESS_THAN = IS_LESS_THAN;
var IS_NOT = binary('is not');
exports.IS_NOT = IS_NOT;
var IS_NOT_EMPTY = unary('is not empty');
exports.IS_NOT_EMPTY = IS_NOT_EMPTY;
var IS_ONE_OF = binary('is one of');
exports.IS_ONE_OF = IS_ONE_OF;
var IS_TRUE = unary('is true');
exports.IS_TRUE = IS_TRUE;

var REFACTOR_ME_OPERATOR_MAP = _lodash.default.reduce([BEGINS_WITH, CONTAINS, DOES_NOT_BEGIN_WITH, DOES_NOT_CONTAIN, DOES_NOT_END_WITH, ENDS_WITH, HAS_NOT_SELECTED, HAS_SELECTED, IS, IS_EMPTY, IS_FALSE, IS_GREATER_THAN, IS_LESS_THAN, IS_NOT, IS_NOT_EMPTY, IS_ONE_OF, IS_TRUE], function (accumulator, operator) {
  return _lodash.default.set(accumulator, operator.operator, operator);
}, {});

var names = function names() {
  for (var _len = arguments.length, operators = new Array(_len), _key = 0; _key < _len; _key++) {
    operators[_key] = arguments[_key];
  }

  return _lodash.default.map(operators, 'operator');
};

exports.names = names;

var getOperator = function getOperator(operator) {
  var structuredOperator = _lodash.default.has(operator, 'operator') ? operator : REFACTOR_ME_OPERATOR_MAP[operator];

  if (!structuredOperator) {
    throw new Error("Unknown operator  \"".concat(operator, "\""));
  }

  return structuredOperator;
};

exports.getOperator = getOperator;
//# sourceMappingURL=operators.js.map
