"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.evaluate = evaluate;
exports.getValidOperators = exports.operatorNotSupported = exports.supportedRightTypes = exports.isOperationSupported = exports.OPERATOR_TYPE_SUPPORT = exports.default = void 0;

var _lodash = _interopRequireDefault(require("lodash"));

var operators = _interopRequireWildcard(require("../operators"));

var _is = _interopRequireDefault(require("./is"));

var _isNot = _interopRequireDefault(require("./is-not"));

var _isTrue = _interopRequireDefault(require("./is-true"));

var _isFalse = _interopRequireDefault(require("./is-false"));

var _isEmpty = _interopRequireDefault(require("./is-empty"));

var _isNotEmpty = _interopRequireDefault(require("./is-not-empty"));

var _hasSelected = _interopRequireDefault(require("./has-selected"));

var _hasNotSelected = _interopRequireDefault(require("./has-not-selected"));

var _isLessThan = _interopRequireDefault(require("./is-less-than"));

var _isGreaterThan = _interopRequireDefault(require("./is-greater-than"));

var _isOneOf = _interopRequireDefault(require("./is-one-of"));

var _contains = _interopRequireDefault(require("./contains"));

var _doesNotContain = _interopRequireDefault(require("./does-not-contain"));

var _beginsWith = _interopRequireDefault(require("./begins-with"));

var _endsWith = _interopRequireDefault(require("./ends-with"));

var _doesNotBeginWith = _interopRequireDefault(require("./does-not-begin-with"));

var _doesNotEndWith = _interopRequireDefault(require("./does-not-end-with"));

var _OPERATORS;

function _interopRequireWildcard(obj) { if (obj && obj.__esModule) { return obj; } else { var newObj = {}; if (obj != null) { for (var key in obj) { if (Object.prototype.hasOwnProperty.call(obj, key)) { var desc = Object.defineProperty && Object.getOwnPropertyDescriptor ? Object.getOwnPropertyDescriptor(obj, key) : {}; if (desc.get || desc.set) { Object.defineProperty(newObj, key, desc); } else { newObj[key] = obj[key]; } } } } newObj.default = obj; return newObj; } }

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

function _defineProperty(obj, key, value) { if (key in obj) { Object.defineProperty(obj, key, { value: value, enumerable: true, configurable: true, writable: true }); } else { obj[key] = value; } return obj; }

var OPERATORS = (_OPERATORS = {}, _defineProperty(_OPERATORS, operators.IS, _is.default), _defineProperty(_OPERATORS, operators.IS_NOT, _isNot.default), _defineProperty(_OPERATORS, operators.IS_TRUE, _isTrue.default), _defineProperty(_OPERATORS, operators.IS_FALSE, _isFalse.default), _defineProperty(_OPERATORS, operators.IS_EMPTY, _isEmpty.default), _defineProperty(_OPERATORS, operators.IS_NOT_EMPTY, _isNotEmpty.default), _defineProperty(_OPERATORS, operators.HAS_SELECTED, _hasSelected.default), _defineProperty(_OPERATORS, operators.HAS_NOT_SELECTED, _hasNotSelected.default), _defineProperty(_OPERATORS, operators.IS_LESS_THAN, _isLessThan.default), _defineProperty(_OPERATORS, operators.IS_GREATER_THAN, _isGreaterThan.default), _defineProperty(_OPERATORS, operators.IS_ONE_OF, _isOneOf.default), _defineProperty(_OPERATORS, operators.CONTAINS, _contains.default), _defineProperty(_OPERATORS, operators.DOES_NOT_CONTAIN, _doesNotContain.default), _defineProperty(_OPERATORS, operators.BEGINS_WITH, _beginsWith.default), _defineProperty(_OPERATORS, operators.ENDS_WITH, _endsWith.default), _defineProperty(_OPERATORS, operators.DOES_NOT_BEGIN_WITH, _doesNotBeginWith.default), _defineProperty(_OPERATORS, operators.DOES_NOT_END_WITH, _doesNotEndWith.default), _OPERATORS);
var _default = OPERATORS;
/*
 * @return
 * {
 *   [IS]: {
 *     [TEXT]: [TEXT, ...]
 *     [USER]: [USER, ...]
 *   }
 * }
 */

exports.default = _default;

var OPERATOR_TYPE_SUPPORT = _lodash.default.reduce(OPERATORS, function (accumulator, operatorTypeMap, operatorName) {
  _lodash.default.forEach(operatorTypeMap, function (rightTypeEvaluators, leftTypeName) {
    var keyPath = [operatorName, leftTypeName];

    var rightTypesSet = _lodash.default.concat(_lodash.default.get(accumulator, keyPath, []), _lodash.default.keys(rightTypeEvaluators));

    _lodash.default.set(accumulator, keyPath, _lodash.default.uniq(rightTypesSet));
  });

  return accumulator;
}, {});

exports.OPERATOR_TYPE_SUPPORT = OPERATOR_TYPE_SUPPORT;

var isOperationSupported = function isOperationSupported(structuredOperator, leftDataType, rightDataType) {
  var keyPath = operators.isUnary(structuredOperator) ? [structuredOperator.operator, leftDataType] : [structuredOperator.operator, leftDataType, rightDataType];
  return _lodash.default.isFunction(_lodash.default.get(OPERATORS, keyPath));
};

exports.isOperationSupported = isOperationSupported;

var supportedRightTypes = function supportedRightTypes(operator, leftDataType) {
  return _lodash.default.get(OPERATOR_TYPE_SUPPORT, [operator.operator, leftDataType], []);
};

exports.supportedRightTypes = supportedRightTypes;

function evaluate(structuredOperator, leftDataType, leftValue, rightDataType, rightValue) {
  if (operators.isUnary(structuredOperator)) {
    return evaluateUnaryOperation(structuredOperator, leftDataType, leftValue);
  }

  return evaluateBinaryOperation(structuredOperator, leftDataType, leftValue, rightDataType, rightValue);
}

function evaluateBinaryOperation(operator, leftDataType, leftValue, rightDataType, rightValue) {
  var keyPath = [operator.operator, leftDataType, rightDataType];
  var operatorFn = _lodash.default.get(OPERATORS, keyPath) || operatorNotSupported.apply(void 0, keyPath);
  return operatorFn(leftValue, rightValue);
}

function evaluateUnaryOperation(operator, leftDataType, leftValue) {
  var keyPath = [operator.operator, leftDataType];
  var operatorFn = _lodash.default.get(OPERATORS, keyPath) || operatorNotSupported.apply(void 0, keyPath);
  return operatorFn(leftValue);
}

var operatorNotSupported = function operatorNotSupported(operator, leftDataType, rightDataType) {
  throw new Error("Operator ".concat(operator.operator, " not supported for left=").concat(leftDataType, ", right=").concat(rightDataType));
};

exports.operatorNotSupported = operatorNotSupported;

var getValidOperators = function getValidOperators(type) {
  var types = _lodash.default.isArray(type) ? type : [type];

  var typeStrings = _lodash.default.flatMap(types, function (type) {
    return _lodash.default.isString(type) ? type : _lodash.default.get(type, 'CAN_COERCE_TO');
  });

  var valid = _lodash.default.reduce(OPERATORS, function (matches, operator, key) {
    if (_lodash.default.intersection(_lodash.default.keys(operator), typeStrings).length) {
      matches.push(key);
    }

    return matches;
  }, []);

  return valid;
};

exports.getValidOperators = getValidOperators;
//# sourceMappingURL=index.js.map
